/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.services.management;

import java.net.URL;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanAttributeInfo;
import javax.management.ObjectName;

import org.ow2.petals.jmx.JMXClient;
import org.ow2.petals.tools.webconsole.services.PetalsService;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.w3c.dom.Document;

/**
 * @author aruffie
 * This interface provide you to
 * installagement
 * services on the bus
 */
public interface ManagementService extends PetalsService {

    public static final String SHUTDOWN = "Shutdown";
    public static final String STARTED = "Started";
    public static final String STOPPED = "Stopped";
    public static final String UNKNOWN = "Unknown";
    
    // components services
    public void installComponent(final URL fileURL, final Map<MBeanAttributeInfo, Object> attributes, final String loadedInstaller) throws PetalsServiceException;
    public void uninstallComponent(final String componentName) throws PetalsServiceException;
    public void shutdownComponent(final String componentName) throws PetalsServiceException;
    public void startComponent(final String componentName) throws PetalsServiceException;
    public void stopComponent(final String componentName) throws PetalsServiceException;
    public Map<MBeanAttributeInfo, Object> recoverRuntimeAttributes(final String componentName) throws PetalsServiceException;
    public void setRuntimeAttributes(final Map<MBeanAttributeInfo, Object> attributes,final String componentName) throws PetalsServiceException;
        
    // service assemblies services
    public void installServiceAssembly(final URL fileURL) throws PetalsServiceException;
    public void uninstallServiceAssembly(final String saName) throws PetalsServiceException;
    public void shutdownServiceAssembly(final String saName) throws PetalsServiceException; 
    public void startServiceAssembly(final String saName) throws PetalsServiceException;
    public void stopServiceAssembly(final String saName) throws PetalsServiceException;
    
    // shared libraries services
    public void installSharedLibrary(final URL sharedLibraryURL) throws PetalsServiceException;
    public void uninstallSharedLibrary(final String sharedLibraryName) throws PetalsServiceException;
    
    // Inventories services
    public String[] serviceAssembliesInventory() throws PetalsServiceException;
    public ObjectName[] componentsInventory() throws PetalsServiceException;
    public ObjectName[] enginesInventory() throws PetalsServiceException;
    public String[] sharedLibrariesInventory() throws PetalsServiceException;
    
    // other services enable
    public JMXClient getJmxClient() throws PetalsServiceException;
    public void setJmxClient(JMXClient jmxClient) throws PetalsServiceException;
    public Set<Map<String, String>> getTopology() throws PetalsServiceException;
    public String getComponentState(String componentName) throws PetalsServiceException;
    public List<Map<String, Object>> getAllEndpoints() throws PetalsServiceException;
    public String getServiceAssemblyState(String sa) throws PetalsServiceException;
    public Map<MBeanAttributeInfo, Object> getConfigComponent(final URL fileURL) throws PetalsServiceException;
    public Document getDocument(final String serviceName, final String endpointName) throws PetalsServiceException;
    public String getLoadedInstaller();

    // Force method
    public void forceUnloadInstaller(final String componentName) throws PetalsServiceException;
    public void forceUnDeploy(final String serviceAssemblyName) throws PetalsServiceException;
}
