/**
 * 
 */
package org.ow2.petals.tools.webconsole.uibeans;

import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.MALFORMED_URL_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.PETALS_SERVICE_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.IOEXCEPTION_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.URI_SYNTAX_ERROR;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.management.MBeanAttributeInfo;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.error.IError;
import org.ow2.opensuit.core.error.NonLocalizedError;
import org.ow2.opensuit.core.impl.multiparts.IFileUploadPart;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.petals.tools.webconsole.business.InstallComponentBBean;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;

/**
 * @author aruffie
 * 
 */
public class InstallComponentUIBean {

    private URL fileURL;

    private InstallComponentBBean businessBean;

    private String serverName;
    
    private List<String> listFileRepo;
    
    private String selectedComponent;
    
    private String urlRepo;
    
    private String url;
    
    private String visibility;
    
    private List<String> visibilities;
    
    private boolean rbrowser;
    private boolean rurl;
    private boolean rrepository;
    private boolean validate;
    private String occuredError;
    private String loadedInstaller;
    
    public void upload(final IFileUploadPart file, final String serverName,
            final String domainName, final HttpServletRequest request) throws NonLocalizedError{
        try {
            this.fileURL = businessBean.storeComponent(file, request);
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
        } catch (final URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
        
        this.serverName = serverName;
    }

    public void load(final HttpServletRequest request){
        final Cookie[] cookies = request.getCookies();
        for(final Cookie cookie : cookies){
            if(cookie.getName().equals("visibilityCookie")){
                this.visibility = cookie.getValue();
            }
        }
    }
    
    public String install(final HttpServletRequest request, final String serverName) throws NonLocalizedError{
        this.serverName = serverName;
        final HttpServletResponse response = OpenSuitSession.getCurrentResponse();
        Map<MBeanAttributeInfo, Object> attributes = null;
        ManagementUIBean managementUIBean = (ManagementUIBean) request.getSession().getAttribute("managementUIBean");
        if(visibility.equals("url")){
            try {
                attributes = businessBean.beginInstall(managementUIBean,this.url,this.serverName);
            } catch (MalformedURLException e) {
                this.occuredError = e.getMessage();
                return "error";
            } catch (PetalsServiceException e) {
                this.occuredError = e.getMessage();
                return "error";
            }
            response.addCookie(new Cookie("visibilityCookie","url"));
        } else if(visibility.equals("local")){
            
            if(this.fileURL != null){
                try {
                    attributes = businessBean.beginInstall(managementUIBean,this.fileURL.toString(),this.serverName);
                } catch (final MalformedURLException e) {
                    this.occuredError = e.getMessage();
                    return "error";
                } catch (final PetalsServiceException e) {
                    this.occuredError = e.getMessage();
                    return "error";
                }
                response.addCookie(new Cookie("visibilityCookie","local"));
            }
        }
        this.loadedInstaller = businessBean.getLoadedInstaller();
        managementUIBean.reconnect(request);

        if(attributes != null){
            request.getSession().setAttribute("attributes",attributes);
            return "success";
        } else {
            this.occuredError = "empty installation attributes";
            return "error";
        }
    }

    public InstallComponentUIBean() {
        this.businessBean = new InstallComponentBBean();
        listFileRepo = new ArrayList<String>();
        
        visibilities = loadVisibilities();
        
        rbrowser = false;
        rurl = false;
        rrepository = false;
        
        if(listFileRepo.size() == 0)
        {
            listFileRepo.add("ListFile");
            listFileRepo.add("liste de fichier");
        }
    }
    
    private List<String> loadVisibilities() {
        final List<String> list = new ArrayList<String>();
        
        list.add("local");
        list.add("url");
        
        return list;
    }

    public List<String> getListFileRepo() {
        return listFileRepo;
    }

    public void setListFileRepo(List<String> listFileRepo) {
        this.listFileRepo = listFileRepo;
    }

    public String getSelectedComponent() {
        return selectedComponent;
    }

    public void setSelectedComponent(String selectedComponent) {
        this.selectedComponent = selectedComponent;
    }    

    public String getUrlRepo() {
        return urlRepo;
    }

    public void setUrlRepo(String urlRepo) {
        this.urlRepo = urlRepo;
        
        
        String tmp[] = urlRepo.split(":");    
        
        if(tmp.length != 2){
            throw new IllegalArgumentException("the syntax must be x.x.x.x:x , example 192.168.1.1:8080");
        }else
        {            
        }
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }
    
    public void getCheckRadioButton(){
        if(visibility.equals("local"))
        {
            this.rbrowser = true;
        }
        else if(visibility.equals("url"))
        {
            this.rurl = true;            
        }
        else if(visibility.equals("repository"))
        {
            this.rrepository = true;
        }
    }
        
    public boolean getBrowserVisibility(){
        
        if(visibility.equals("local") && rbrowser == true){
            return true;
        }
        return false;            
    }
    
    public boolean getUrlVisibility(){
        
        if(visibility.equals("url") && rurl == true){
            return true;
        }
        return false;            
    }
    
    public boolean getRepositoryVisibility(){
        
        if(visibility.equals("repository") && rrepository == true){
            return true;
        }
        return false;            
    }    

    public String getVisibility() {
        return visibility;
    }

    public void setVisibility(String visibility) {
        this.visibility = visibility;
    }

    public List<String> getVisibilities() {
        return visibilities;
    }

    public void setVisibilities(List<String> visibilities) {
        this.visibilities = visibilities;
    }

    public boolean isValidate() {
        return validate;
    }

    public void setValidate(boolean validate) {
        this.validate = validate;
    }
    
    public String validateAttributes(final HttpServletRequest request) throws NonLocalizedError{
        final Map<MBeanAttributeInfo, Object> attributes =
            (Map<MBeanAttributeInfo, Object>) request.getSession()
            .getAttribute("attributes");
        final List<String> occuredErrors = new ArrayList<String>();
        for(final MBeanAttributeInfo info : attributes.keySet()){
            final String s= request.getParameter(info.getName());
            
            Object result = null;
            try{
                if(info.getType().equals("int")||
                   info.getType().equals("Int")||
                   info.getType().equals("integer")||
                   info.getType().equals("Integer")){
                    result = Integer.valueOf(s);
                }
                if((info.getType().equals("string"))||
                   (info.getType().equals("String"))||
                   (info.getType().equals("java.lang.String"))){
                    result = s;
                }
                if(info.getType().equals("float")||
                   info.getType().equals("Float")){
                    result = Float.valueOf(s);
                }
                if(info.getType().equals("double")||
                   info.getType().equals("Double")){
                    result = Double.valueOf(s);
                }
                if(info.getType().equals("short")||
                   info.getType().equals("Short")){
                    result = Short.valueOf(s);
                }
                if(info.getType().equals("bool")||
                   info.getType().equals("Bool")||
                   info.getType().equals("boolean")||
                   info.getType().equals("Boolean")){
                    result = Boolean.valueOf(s);
                }
                if(info.getType().equals("char")||
                   info.getType().equals("Char")||
                   info.getType().equals("character")||
                   info.getType().equals("Character")){
                    result = s.toCharArray();
                }
                if(info.getType().equals("byte")||
                   info.getType().equals("Byte")){
                    result = Byte.valueOf(s);
                }
                if(info.getType().equals("long")||
                   info.getType().equals("Long")){
                    result = Long.valueOf(s);
                }
                attributes.put(info, result);
            }catch(final NumberFormatException e){
                occuredErrors.add("The value '"+s+"' isn't appropriate for '"+
                                 info.getName()+"' attribute, must be '"+
                                 info.getType()+"' type");
            }
        }
        if(occuredErrors.isEmpty()){
            if(this.url != null){
                try {
                    businessBean.install(request, url, serverName, attributes, loadedInstaller);
                } catch (final MalformedURLException e) {
                    final NonLocalizedError error = new NonLocalizedError(MALFORMED_URL_ERROR,e.getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                } catch (final IOException e) {
                    throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
                } catch (final URISyntaxException e) {
                    final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                } catch (final PetalsServiceException e) {
                    throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
                }
            } else {
                try {
                    businessBean.install(request, this.fileURL.toString(), serverName, attributes,loadedInstaller);
                } catch (final MalformedURLException e) {
                    final NonLocalizedError error = new NonLocalizedError(MALFORMED_URL_ERROR,e.getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                } catch (final IOException e) {
                    throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
                } catch (final URISyntaxException e) {
                    final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                } catch (final PetalsServiceException e) {
                    throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
                }
            }
            ManagementUIBean managementUIBean = (ManagementUIBean) request.getSession().getAttribute("managementUIBean");
            managementUIBean.reconnect(request);
            return "success";
        }else {
            request.getSession().setAttribute("occuredErrors", occuredErrors);
            return "error";
        }
    }

    public String getOccuredError() {
        return occuredError;
    }
    
    
}
