/**
 * 
 */
package org.ow2.petals.tools.webconsole.uibeans;


import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.error.IError;
import org.ow2.opensuit.core.error.NonLocalizedError;
import org.ow2.opensuit.core.impl.multiparts.IFileUploadPart;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.petals.tools.webconsole.business.InstallServiceAssemblyBBean;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.IOEXCEPTION_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.URI_SYNTAX_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.PETALS_SERVICE_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.MALFORMED_URL_ERROR;

/**
 * @author aruffie
 * 
 */
public class InstallServiceAssemblyUIBean {

    private URL fileURL;

    private InstallServiceAssemblyBBean businessBean;

    private String serverName;
    
    private String url;
    
    private String visibility;
    
    private List<String> visibilities;
    
    private boolean rbrowser;
    private boolean rurl;
    
    public InstallServiceAssemblyUIBean(){
        this.visibilities = loadVisibilities();
        this.businessBean = new InstallServiceAssemblyBBean();
    }
    
    private List<String> loadVisibilities() {
        final List<String> list = new ArrayList<String>();
        
        list.add("local");
        list.add("url");
        
        return list;
    }
    
    public void getCheckRadioButton(){
        if(visibility.equals("local"))
        {
            this.rbrowser = true;
        }
        else if(visibility.equals("url"))
        {
            this.rurl = true;            
        }
        else if(visibility.equals("repository"))
        {
        }
    }
    
    public void load(final HttpServletRequest request){
        final Cookie[] cookies = request.getCookies();
        for(final Cookie cookie : cookies){
            if(cookie.getName().equals("visibilityCookie")){
                this.visibility = cookie.getValue();
            }
        }
    }

    public String getVisibility() {
        return visibility;
    }

    public void setVisibility(String visibility) {
        this.visibility = visibility;
    }

    public List<String> getVisibilities() {
        return visibilities;
    }

    public void setVisibilities(List<String> visibilities) {
        this.visibilities = visibilities;
    }   
    
    public boolean getBrowserVisibility(){
        
        if(visibility.equals("local") && rbrowser == true){
            return true;
        }
        return false;            
    }
    
    public boolean getUrlVisibility(){
        
        if(visibility.equals("url") && rurl == true){
            return true;
        }
        return false;            
    }
    
    public void upload(final IFileUploadPart file, final String serverName,
            final String domainName, final HttpServletRequest request) throws NonLocalizedError {
        try {
            this.fileURL = businessBean.storeServiceAssembly(file, request);
        } catch (final URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
        }
        this.serverName = serverName;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }
    
    public String install(final HttpServletRequest request, final String serverName) throws NonLocalizedError {
        this.serverName = serverName;
        final HttpServletResponse response = OpenSuitSession.getCurrentResponse();
        ManagementUIBean managementUIBean = (ManagementUIBean) request.getSession().getAttribute("managementUIBean");
        if(visibility.equals("url")){
            try {
                businessBean.beginInstall(request, managementUIBean,this.url,this.serverName);
            } catch (final MalformedURLException e) {
                final NonLocalizedError error = new NonLocalizedError(MALFORMED_URL_ERROR,e.getLocalizedMessage());
                error.setType(IError.FUNCTIONAL_ERROR);
                throw error;
            } catch (final PetalsServiceException e) {
                throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
            } catch (final IOException e) {
                throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
            } catch (final URISyntaxException e) {
                final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage());
                error.setType(IError.FUNCTIONAL_ERROR);
                throw error;
            }
            
            response.addCookie(new Cookie("visibilityCookie","url"));
        } else if(visibility.equals("local")){
            
            if(this.fileURL != null){
                try {
                    businessBean.beginInstall(request,managementUIBean,this.fileURL.toString(),this.serverName);
                } catch (final MalformedURLException e) {
                    final NonLocalizedError error = new NonLocalizedError(MALFORMED_URL_ERROR,e.getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                } catch (final PetalsServiceException e) {
                    throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
                } catch (final IOException e) {
                    throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
                } catch (final URISyntaxException e) {
                    final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                }
                
                response.addCookie(new Cookie("visibilityCookie","local"));
            }
        }
        managementUIBean.reconnect(request);
        return "success";
    }
}
