/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.uibeans;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.management.ManagementFactory;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.namespace.QName;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.stream.StreamSource;

import org.objectweb.petals.tools.rmi.common.util.InputStreamForker;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.InterfaceType;
import org.ow2.easywsdl.wsdl.api.Operation;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLImportException;
import org.ow2.opensuit.core.error.IError;
import org.ow2.opensuit.core.error.LocalizedError;
import org.ow2.opensuit.core.error.NonLocalizedError;
import org.ow2.opensuit.core.impl.multiparts.IFileUploadPart;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.petals.tools.webconsole.business.RMIInjectorBBean;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.ow2.petals.tools.webconsole.services.management.ManagementService;
import org.ow2.petals.tools.webconsole.services.management.ManagementServiceFactory;
import org.ow2.petals.tools.webconsole.services.rmiinjector.model.Mep;
import org.ow2.petals.tools.webconsole.services.rmiinjector.model.Property;
import org.ow2.petals.tools.webconsole.to.DomainTO;
import org.ow2.petals.tools.webconsole.to.EndPointTO;
import org.ow2.petals.tools.webconsole.to.ServerTO;
import org.ow2.petals.tools.webconsole.util.ConfigHelper;
import org.ow2.petals.tools.webconsole.util.GeneralHelper;
import org.ow2.petals.tools.webconsole.util.StringHelper;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.DOWNLOAD_ATTACHMENT_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.FACTORY_CONFIGURATION_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.FILE_NOT_FOUND_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.ILLEGAL_ARGUMENT_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.IOEXCEPTION_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.MALFORMED_URL_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.MESSAGING_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.PARSER_CONFIGURATION_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.PETALS_SERVICE_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.RMI_SERVER_CONFIGURATION_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.SAX_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.SELECTED_ENDPOINT_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.TRANSFORMER_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.TRANSFORMER_FACTORY_CONFIGURATION_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.URI_SYNTAX_ERROR;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.WSDLEXCEPTION_ERROR;
/**
 * @author aruffie
 * 
 */
public class RMIInjectorUIBean {

    private List<EndPointTO> endpoints;

    private List<String> operations;

    private String selectedOperation;

    private EndPointTO selectedEndpoint;

    private ServerTO currentServer;

    private String content;

    private String response;

    private List<String> meps;

    private List<Property> mEXProperties;
    
    private List<Property> normalizedProperties;
    
    private Property addedProperty;

    private List<String> selectedMEXProperties;
    
    private List<String> selectedNormalizedProperties;
    
    private List<File> attachments;
    
    private List<String> selectedAttachments;

    private String selectedMep;

    private long timeout;

    private ServerTO rmiServer;

    private RMIInjectorBBean businessInjector;

    private List<String> selectedExchanges;
    
    private boolean rmiComponentAvailable;
    
    private boolean manually;
    
    private boolean performedSend;
    
    public RMIInjectorUIBean() throws NonLocalizedError {
        this.endpoints = new ArrayList<EndPointTO>();
        this.operations = new ArrayList<String>();
        this.selectedAttachments = new ArrayList<String>();
        this.meps = new ArrayList<String>();
        this.mEXProperties = new ArrayList<Property>();
        this.normalizedProperties = new ArrayList<Property>();
        this.attachments = new ArrayList<File>();
        this.selectedMEXProperties = new ArrayList<String>();
        this.selectedNormalizedProperties = new ArrayList<String>();
        try {
            this.businessInjector = new RMIInjectorBBean();
        } catch (final PetalsServiceException e) {
            throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
        }
        this.timeout = 30000;
        initmeps();
    }

    /**
     * Initialize map
     */
    private void initmeps() {
        this.meps.add(Mep.IN_ONLY.value());
        this.meps.add(Mep.IN_OPTIONAL_OUT.value());
        this.meps.add(Mep.IN_OUT.value());
        this.meps.add(Mep.ROBUST_IN_ONLY.value());
    }

    public void load(final HttpServletRequest request) throws NonLocalizedError{
        
        
        if(!this.performedSend){
            this.operations.clear();
            this.endpoints.clear();
            this.operations.add("no operation selected operations selected");
            final EndPointTO endpoint = new EndPointTO(null, null, "no endpoint selected", null, null,
                    null);
            this.endpoints.add(endpoint);
            this.selectedEndpoint = endpoints.get(0);
            this.selectedOperation = operations.get(0);
        }
        
        this.performedSend=false;
        
        final ManagementUIBean managementBean = (ManagementUIBean) request.getSession()
                .getAttribute("managementUIBean");
        if(managementBean != null){
            final String serverName = request.getParameter("serverName");
            if((serverName != null) && (!serverName.equals(""))){
                final DomainTO domain = managementBean.getDomain();
                this.currentServer = domain.findServer(serverName);
                
                final List<EndPointTO> serverEndpoints = currentServer.getAllEndpoints();
                final List<String> endpointAlreadyPresent = new ArrayList<String>();
                for(final EndPointTO edp : this.endpoints){
                    endpointAlreadyPresent.add(edp.getName());
                }
                for(final EndPointTO serverEndpoint : serverEndpoints){
                    if(!endpointAlreadyPresent.contains(serverEndpoint.getName())){
                        this.endpoints.add(serverEndpoint);
                    }
                }
            }
            
        }
    }

    public List<EndPointTO> getEndpoints() {
        if(this.endpoints.isEmpty()){
            this.endpoints.add(new EndPointTO("no component","no interface","no endpoint","no server","no service","no type"));
        }
        return endpoints;
    }

    public void setEndpoints(List<EndPointTO> endpoints) {
        this.endpoints = endpoints;
    }

    public String getSelectedEndpoint() {
        if (!this.endpoints.isEmpty() && this.selectedEndpoint == null) {
            this.selectedEndpoint = this.endpoints.get(0);
        }
        return this.selectedEndpoint.getName();
    }

    public void setSelectedEndpoint(String nameEndpoint){
        if(!this.manually){
            this.selectedEndpoint = findEndpoint(nameEndpoint);
        } else {
            final EndPointTO knowedEndpoint = findEndpoint(nameEndpoint);
            this.selectedEndpoint = new EndPointTO();
            this.selectedEndpoint.setName(nameEndpoint);
            if(knowedEndpoint != null){
                this.selectedEndpoint.setService(knowedEndpoint.getService());
                this.selectedEndpoint.setInterfaceName(knowedEndpoint.getInterfaceName());
            }
        }
    }

    public void loadOperations(final HttpServletRequest request) throws NonLocalizedError{
        this.operations.clear();
        operations.add("no operation selected");
        if(this.selectedEndpoint.getName().equals("no endpoint selected")){
            return;
        }
        
        ManagementService service = null;
        
        
        if(this.currentServer != null){
            Document wsdlDocument = null;
            try {
                service = (ManagementService) ManagementServiceFactory.getInstance().getService(
                        this.currentServer.getHost(),
                        Integer.parseInt(this.currentServer.getJmxPort()),
                        this.currentServer.getJmxLogin(), this.currentServer.getJmxPassword());
                this.rmiComponentAvailable = true;
                wsdlDocument = service.getDocument(this.selectedEndpoint.getService(),
                        this.selectedEndpoint.getName());
            } catch (final NumberFormatException e) {
                e.printStackTrace();
            } catch (final PetalsServiceException e){
                this.rmiComponentAvailable = false;
                e.printStackTrace();
            }
            
            Description description = null;
            if(wsdlDocument != null){
                try {
                    description = GeneralHelper.readWSDL(wsdlDocument);
                    for (final InterfaceType interfaze : description.getInterfaces()) {
                        for (final Operation operation : interfaze.getOperations()) {
                            operations.add(operation.getQName().getLocalPart());
                        }
                    }
                } catch (final WSDLException e) {
                    if (e.getCause().getCause() instanceof WSDLImportException) {
                        operations.add("WSDLImportException: " + e.getCause().getCause().getMessage());
                    } else {
                        throw new NonLocalizedError(WSDLEXCEPTION_ERROR,e.getLocalizedMessage(),e);
                    }
                } catch (final MalformedURLException e) {
                    final NonLocalizedError error = new NonLocalizedError(MALFORMED_URL_ERROR,e.getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                } catch (final URISyntaxException e) {
                    final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage());
                    error.setType(IError.FUNCTIONAL_ERROR);
                    throw error;
                }
            }
            
            /*
             * If the current server where  the RMI component isn't etablish,
             * try to find it
             */
        } else {
            this.load(request);
        }
    }

    public List<String> getOperations() {
        if(this.operations.isEmpty()){
            this.operations.add("no operation selected");
        }
        return operations;
    }

    public void setOperations(List<String> operations) {
        this.operations = operations;
    }

    public String getSelectedOperation() {
        return selectedOperation;
    }

    public void setSelectedOperation(String selectedOperation) {
        this.selectedOperation = selectedOperation;
    }

    public ServerTO getCurrentServer() {
        return currentServer;
    }

    public void setCurrentServer(ServerTO currentServer) {
        this.currentServer = currentServer;
    }

    private EndPointTO findEndpoint(final String ep) {
        for (final EndPointTO endpoint : this.endpoints) {
            if (endpoint.getName().equals(ep)) {
                return endpoint;
            }
        }
        return null;
    }

    public List<String> getEndpointsNames() {
        final List<String> endpointsNames = new ArrayList<String>();
        for (final EndPointTO endpoint : this.endpoints) {
            endpointsNames.add(endpoint.getName());
        }
        return endpointsNames;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    public void send(final HttpServletRequest request) throws NonLocalizedError {
        this.performedSend = true;
        if(this.rmiServer == null){
            final NonLocalizedError error = new NonLocalizedError(RMI_SERVER_CONFIGURATION_ERROR," rmi server unavailable:\n - check if the petals-se-rmi is installed over a petals node \n - check if you have declared the available petals-se-rmi, in webconsole configuration file");
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
        
        MessageExchange currentExchange = null;
        if(selectedEndpoint.getName().equals("no endpoint selected")){
            final NonLocalizedError error = new NonLocalizedError(SELECTED_ENDPOINT_ERROR," you try to send a message to 'no endpoint selected' , remove this default value");
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } else {
            try{
             
            QName qnOperation = null;
            QName qnService = null;
            String sEndpoint = null;
            if(StringHelper.isNotNullAndNotEmpty(this.selectedOperation)){
                qnOperation = QName.valueOf(selectedOperation);
            }
            if(StringHelper.isNotNullAndNotEmpty(this.selectedEndpoint.getService())){
                qnService = QName.valueOf(this.selectedEndpoint.getService());
            }
            if(StringHelper.isNotNullAndNotEmpty(this.selectedEndpoint.getName())){
                sEndpoint = this.selectedEndpoint.getName();
            }
            currentExchange = businessInjector.send(attachments, selectedMep, rmiServer,
                    qnOperation, timeout, QName.valueOf(selectedEndpoint
                            .getInterfaceName()), qnService, sEndpoint, this.mEXProperties, this.normalizedProperties, content);
            } catch (final IllegalArgumentException e){            
                final NonLocalizedError error = new NonLocalizedError(ILLEGAL_ARGUMENT_ERROR,e.getLocalizedMessage());
                error.setType(IError.FUNCTIONAL_ERROR);
                throw error;
            } catch (final PetalsServiceException e) {
                throw new NonLocalizedError(PETALS_SERVICE_ERROR,e.getLocalizedMessage(),e);
            }
        }

        if (currentExchange != null) {         
            if(currentExchange.getError() != null){
                String error = null;
                final Exception e = currentExchange.getError();
                try{
                    final File tmpfile = File.createTempFile("tmp", "error");
                    final PrintWriter writer = new PrintWriter(tmpfile);
                    e.printStackTrace(writer);
                    writer.flush();
                    writer.close();
                    final BufferedReader reader = new BufferedReader(new FileReader(tmpfile));
                    StringWriter swriter = new StringWriter();
                    String line = null;
                    do{
                        line = reader.readLine();
                        if(line != null){
                            swriter.write(line);
                        }
                    }while(line != null);
                    reader.close();
                    error = swriter.toString();
                    swriter.close();
                } catch(final IOException ex){
                    throw new NonLocalizedError(IOEXCEPTION_ERROR,ex.getLocalizedMessage(),ex);
                }
                this.response = e.getMessage()+"\n"+error;
            } else if(currentExchange.getFault() != null){
                final Fault f = currentExchange.getFault();
                try {
                    final InputStreamForker forker = StringHelper.fork(f.getContent());
                    this.response = StringHelper.prettyPrintWithSource(new StreamSource(forker.getInputStreamOne()));
                    f.setContent(new StreamSource(forker.getInputStreamTwo()));
                } catch (final TransformerFactoryConfigurationError e) {
                    throw new NonLocalizedError(TRANSFORMER_FACTORY_CONFIGURATION_ERROR,e.getLocalizedMessage(),e);
                } catch (final IOException e) {
                    throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
                } catch (final TransformerException e) {
                    throw new NonLocalizedError(TRANSFORMER_ERROR,e.getLocalizedMessage(),e);
                } catch (final MessagingException e) {
                    throw new NonLocalizedError(MESSAGING_ERROR,e.getLocalizedMessage(),e);
                }
            } else if(currentExchange.getMessage("OUT")!= null){
                final NormalizedMessage nm = currentExchange.getMessage("OUT");
                try {
                    final InputStreamForker forker = StringHelper.fork(nm.getContent());
                    this.response = StringHelper.prettyPrintWithSource(new StreamSource(forker.getInputStreamOne()));
                    nm.setContent(new StreamSource(forker.getInputStreamTwo()));
                } catch (final TransformerFactoryConfigurationError e) {
                    throw new NonLocalizedError(TRANSFORMER_FACTORY_CONFIGURATION_ERROR,e.getLocalizedMessage(),e);
                } catch (final IOException e) {
                    throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
                } catch (final TransformerException e) {
                    throw new NonLocalizedError(TRANSFORMER_ERROR,e.getLocalizedMessage(),e);
                } catch (MessagingException e) {
                    throw new NonLocalizedError(MESSAGING_ERROR,e.getLocalizedMessage(),e);
                }
            } else if(this.selectedMep.equals(Mep.IN_ONLY.value())){
                this.response = "Nothing content returned because "+Mep.IN_ONLY.value()+" pattern used";
            }
        }
    }

    public String getResponse() {
        return response;
    }

    public void setResponse(String response) {
        this.response = response;
    }

    public List<String> getMeps() {
        return meps;
    }

    public void setMeps(List<String> meps) {
        this.meps = meps;
    }

    public String getSelectedMep() {
        return selectedMep;
    }

    public void setSelectedMep(String selectedMep) {
        this.selectedMep = selectedMep;
    }

    public String getTimeout() {
        return Long.toString(timeout);
    }

    public void setTimeout(String timeout) {
        this.timeout = Long.parseLong(timeout);
    }

    public String getSelectedService() {
        if(StringHelper.isNullOrEmpty(this.selectedEndpoint.getService())){
            final EndPointTO knowedEndpoint = findEndpoint(this.selectedEndpoint.getName());
            if(knowedEndpoint != null){
                this.selectedEndpoint.setService(knowedEndpoint.getService());             
            }
        }
        return this.selectedEndpoint.getService();
    }

    public void setSelectedService(final String service) {
        if(this.manually){
            if(this.selectedEndpoint == null){
                this.selectedEndpoint = new EndPointTO();
            }
            this.selectedEndpoint.setService(service);
        }
    }
    
    public void setSelectedInterface(final String _interface) {
        if(this.manually){
            if(this.selectedEndpoint == null){
                this.selectedEndpoint = new EndPointTO();
            }
            this.selectedEndpoint.setInterfaceName(_interface);
        }
    }
    
    public String getSelectedInterface() {
        if(StringHelper.isNullOrEmpty(this.selectedEndpoint.getInterfaceName())){
            final EndPointTO knowedEndpoint = findEndpoint(this.selectedEndpoint.getName());
            if(knowedEndpoint != null){
                this.selectedEndpoint.setInterfaceName(knowedEndpoint.getInterfaceName());             
            }
        }
        return this.selectedEndpoint.getInterfaceName();
    }

    /**
     * Getter for properties, also allow to validate the content
     * 
     * @param selectedProperties
     */
    public void setSelectedProperties(String selectedProperties) {
        if (StringHelper.isNotNullAndNotEmpty(selectedProperties)) {
            selectedProperties = selectedProperties.replaceAll("\r", "");
            final String[] allProps = selectedProperties.split("\n|;");
            for (final String prop : allProps) {
                final String[] propParts = prop.split("=");
                if (propParts.length == 2) {
                    final Property property = new Property();
                    property.setName(propParts[0]);
                    property.setValue(propParts[1]);
                    mEXProperties.add(property);
                } else {
                    mEXProperties.clear();
                    final Property property = new Property();
                    property.setName("Error: bad pattern for property ");
                    property
                            .setValue(" Must be \'key=value\' , and not \'key==value\' or \';;\' ...");
                    mEXProperties.add(property);
                    return;
                }
            }
        }
    }

    public String getSelectedProperties() {
        StringBuilder builder = new StringBuilder();
        for (Property prop : mEXProperties) {
            builder.append(prop.getName() + "=" + prop.getValue() + ";");
        }
        return builder.toString();
    }

    public void upload(final IFileUploadPart file, final HttpServletRequest request)
            throws NonLocalizedError {

        try {
            attachments.add(businessInjector.storeAttachement(file, request));
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
        } catch (final URISyntaxException e) {
            final NonLocalizedError error = new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
    }

    public String attach() throws NonLocalizedError {
        final File currentFile = attachments.get(attachments.size() - 1);

        /*
         * If file too voluminous return error
         */
        FileInputStream in = null;
        try{
                in = new FileInputStream(currentFile);
        } catch (final FileNotFoundException e){       
            final NonLocalizedError error = new NonLocalizedError(FILE_NOT_FOUND_ERROR,e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        }
        
        try {
            
            final byte[] data = new byte[in.available()];
            
        } catch (final java.lang.OutOfMemoryError e) {
            attachments.remove(currentFile);
            this.response = "Error: Attachment " + currentFile.getName() + " too voluminous";
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
        } finally {
            try {
                in.close();
            } catch (final IOException e) {
                throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
            }
        }
        return "success";
    }
    
    

    public List<File> getAttachments() {
        return attachments;
    }

    public void setAttachments(List<File> attachments) {
        this.attachments = attachments;
    }

    /**
     * Allow to delete selected Attachments
     * @return String
     */
    public String deleteAttachements() {
        final List<File> deletedAttachments = new ArrayList<File>();
        for (final String fileName : selectedAttachments) {
            for (final File attachement : attachments) {
                if (attachement.getName().equals(fileName)) {
                    deletedAttachments.add(attachement);
                }
            }
        }
        
        this.attachments.removeAll(deletedAttachments);
        return "success";
    }

    public List<MessageExchange> getExchanges() {
        return businessInjector.getExchanges();
    }

    
    public List<String> getSelectedExchanges() {
        return selectedExchanges;
    }

    public void setSelectedExchanges(final List<String> selectedExchanges) {
        this.selectedExchanges = selectedExchanges;
    }

    /**
     * Delete all selected responses
     * @throws NonLocalizedError 
     */
    public void deleteSelectedExchanges() throws NonLocalizedError {

        final List<MessageExchange> deletedExchanges = new ArrayList<MessageExchange>();
        final List<MessageExchange> businessExchanges = this.businessInjector.getExchanges();
        final List<String> deletedAttachments = new ArrayList<String>();
        /*
         * Recover all deleted responses selected and check if these are
         * presente in the business layer.
         */
        for (final String exchangeId : selectedExchanges) {
            for (final MessageExchange bExchange : businessExchanges) {
                if (bExchange.getExchangeId().equals(exchangeId)) {
                    NormalizedMessage nm = bExchange.getMessage("IN");
                    if(nm != null){
                        final Set<?> set = nm.getAttachmentNames();
                        final Iterator<?> it = set.iterator();
                        while(it.hasNext()){
                            final String attachmentId = (String) it.next();
                            deletedAttachments.add(attachmentId);
                        }
                    }
                    nm = null;
                    
                    nm = bExchange.getMessage("OUT");
                    if(nm != null){
                        final Set<?> set = nm.getAttachmentNames();
                        final Iterator<?> it = set.iterator();
                        while(it.hasNext()){
                            final String attachmentId = (String) it.next();
                            deletedAttachments.add(attachmentId);
                        }
                    }
                    nm = null;
                    
                    nm = bExchange.getFault();
                    if(nm != null){
                        final Set<?> set = nm.getAttachmentNames();
                        final Iterator<?> it = set.iterator();
                        while(it.hasNext()){
                            final String attachmentId = (String) it.next();
                            deletedAttachments.add(attachmentId);
                        }
                    }
                    
                    deletedExchanges.add(bExchange);
                }
            }
        }

        /*
         * Delete all responses and displayed attachments into interface
         */
        businessExchanges.removeAll(deletedExchanges);
        final List<File> removeAttachment = new ArrayList<File>();
        for(final File attach : this.attachments){
            if(deletedAttachments.contains(attach.getName())){
                removeAttachment.add(attach);
            }
        }
        this.attachments.removeAll(removeAttachment);
        
        /*
         * Delete selected attachments into the repository
         */
        try {
            this.businessInjector.removeAttachments(deletedAttachments, OpenSuitSession.getCurrentRequest());
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
        } catch (final URISyntaxException e) {
            throw new NonLocalizedError(URI_SYNTAX_ERROR,e.getLocalizedMessage(),e);
        }
    }
    
    public String selectAllExchanges(){
        for(final MessageExchange exchange : this.businessInjector.getExchanges()){
            this.selectedExchanges.add(exchange.getExchangeId());
        }
        return "success";
    }

    public boolean isReachableRMIComponent() throws NonLocalizedError{
        if(this.rmiServer != null){
            try {
                this.rmiComponentAvailable = this.businessInjector.checkRMIComponentAvailable(this.rmiServer);
            } catch (final NumberFormatException e) {
                // Dot nothing just display rmi component not available
            } catch (final PetalsServiceException e) {
             // Dot nothing just display rmi component not available
            }
        }
        return this.rmiComponentAvailable;
    }
    
    public boolean isDeclaredRMIComponent() throws NonLocalizedError {
        
        try {
            this.rmiServer = ConfigHelper.findRMIServer(
                    ManagementUIBean.class.getClassLoader().getResource(
                    "webconsole.xml"));
            if(this.rmiServer != null){
                return true;
            } else {
                return false;
            }
        } catch (final MalformedURLException e) {
            final NonLocalizedError error = new NonLocalizedError(MALFORMED_URL_ERROR,e.getLocalizedMessage());
            error.setType(IError.FUNCTIONAL_ERROR);
            throw error;
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
        } catch (final SAXException e) {
            throw new NonLocalizedError(SAX_ERROR,e.getLocalizedMessage(),e);
        } catch (final ParserConfigurationException e) {
            throw new NonLocalizedError(PARSER_CONFIGURATION_ERROR,e.getLocalizedMessage(),e);
        } catch (final FactoryConfigurationError e) {
            throw new NonLocalizedError(FACTORY_CONFIGURATION_ERROR,e.getLocalizedMessage(),e);
        }
    }

    public boolean isManually() {
        return manually;
    }

    public void setManually(boolean manually) {
        this.manually = manually;
    }
    
    /**
     * Allow to delete selected properties in the 
     * message exchange properties table
     */
    public String deleteMessageExchangeProperties(){
        final List<Property> deletedProperties = new ArrayList<Property>();
        for(final String p : this.selectedMEXProperties){
            for(final Property prop : this.mEXProperties){
                if(prop.getName().equals(p)){
                    deletedProperties.add(prop);
                }
            }
        }
        this.mEXProperties.removeAll(deletedProperties);
        return "success";
    }
    
    /**
     * Allow to delete selected properties in the 
     * normalized message properties table
     */
    public String deleteNormalizedProperties(){
        final List<Property> deletedProperties = new ArrayList<Property>();
        for(final String p : this.selectedNormalizedProperties){
            for(final Property prop : this.normalizedProperties){
                if(prop.getName().equals(p)){
                    deletedProperties.add(prop);
                }
            }
        }
        this.normalizedProperties.removeAll(deletedProperties);
        return "success";
    }

    public List<Property> getMEXProperties() {
        return this.mEXProperties;
    }

    public void setMEXProperties(List<Property> properties) {
        this.mEXProperties = properties;
    }

    public List<String> getSelectedMEXProperties() {
        return selectedMEXProperties;
    }

    public void setSelectedMEXProperties(List<String> selectedMEXProperties) {
        this.selectedMEXProperties = selectedMEXProperties;
    }

    public Property getAddedProperty() {
        return this.addedProperty;
    }

    public void setAddedProperty(Property addedProperty) {
        this.addedProperty = addedProperty;
    }

    /**
     * Add message exchange property
     * @return String
     */
    public String addMEXProperty(){
        final Property p = new Property();
        p.setName(this.addedProperty.getName());
        p.setValue(this.addedProperty.getValue());
        this.mEXProperties.add(p);
        this.addedProperty = null;
        return "success";
    }
    
    /**
     * Add normalized message property
     * @return String
     */
    public String addNormalizedProperty(){
        final Property p = new Property();
        p.setName(this.addedProperty.getName());
        p.setValue(this.addedProperty.getValue());
        this.normalizedProperties.add(p);
        this.addedProperty = null;
        return "success";
    }
    
    /**
     * Initialize the add of property
     */
    public void loadAddProperty(){
        this.addedProperty = new Property();
    }

    public List<Property> getNormalizedProperties() {
        return normalizedProperties;
    }

    public void setNormalizedProperties(List<Property> normalizedProperties) {
        this.normalizedProperties = normalizedProperties;
    }

    public List<String> getSelectedNormalizedProperties() {
        return selectedNormalizedProperties;
    }

    public void setSelectedNormalizedProperties(List<String> selectedNormalizedProperties) {
        this.selectedNormalizedProperties = selectedNormalizedProperties;
    }

    public List<String> getSelectedAttachments() {
        return selectedAttachments;
    }

    public void setSelectedAttachments(List<String> selectedAttachments) {
        this.selectedAttachments = selectedAttachments;
    }
    
    public String getNMContent(final MessageExchange exchange, final String message) throws NonLocalizedError{
        final NormalizedMessage nm= exchange.getMessage(message);
        try {
            final InputStreamForker forker = StringHelper.fork(nm.getContent());
            final String result = "\n"+StringHelper.prettyPrintWithSource(new StreamSource(forker.getInputStreamOne()));
            nm.setContent(new StreamSource(forker.getInputStreamTwo()));
            return result;
        }catch (final TransformerFactoryConfigurationError e) {
            throw new NonLocalizedError(TRANSFORMER_FACTORY_CONFIGURATION_ERROR,e.getLocalizedMessage(),e);
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
        } catch (final TransformerException e) {
            throw new NonLocalizedError(TRANSFORMER_ERROR,e.getLocalizedMessage(),e);
        } catch (final MessagingException e) {
            throw new NonLocalizedError(MESSAGING_ERROR,e.getLocalizedMessage(),e);
        }
    }
    
    public String splitPattern(final URI uri){
        final String[] splittedPattern = uri.toString().split("/");
        if(splittedPattern != null){
            return splittedPattern[splittedPattern.length-1];
        }
        return null;
    }
    
    public String getFaultContent(final MessageExchange exchange) throws NonLocalizedError{
        final Fault fault = exchange.getFault();
        try {
            final InputStreamForker forker = StringHelper.fork(fault.getContent());
            final String result = "\n"+StringHelper.prettyPrintWithSource(new StreamSource(forker.getInputStreamOne()));
            fault.setContent(new StreamSource(forker.getInputStreamTwo()));
            return result;
        }catch (final TransformerFactoryConfigurationError e) {
            throw new NonLocalizedError(TRANSFORMER_FACTORY_CONFIGURATION_ERROR,e.getLocalizedMessage(),e);
        } catch (final IOException e) {
            throw new NonLocalizedError(IOEXCEPTION_ERROR,e.getLocalizedMessage(),e);
        } catch (final TransformerException e) {
            throw new NonLocalizedError(TRANSFORMER_ERROR,e.getLocalizedMessage(),e);
        } catch (final MessagingException e) {
            throw new NonLocalizedError(MESSAGING_ERROR,e.getLocalizedMessage(),e);
        }
    }
    
    public String getMessageError(final MessageExchange exchange){
        return "\n"+exchange.getError().getMessage();
    }
    
    public List<Property> getMEXProperties(final MessageExchange exchange){
        final List<Property> properties = new ArrayList<Property>();
        final Set<?> set = exchange.getPropertyNames();
        final Iterator<?> it = set.iterator();
        while(it.hasNext()){
            final String pName = (String) it.next();
            final Property prop = new Property();
            prop.setName(pName);
            if(exchange.getProperty(pName) != null){
                prop.setValue(exchange.getProperty(pName).toString());
            } else {
                prop.setValue("null");
            }
            properties.add(prop);
        }
        return properties;
    }
    
    public List<Property> getNMProperties(final MessageExchange exchange, final String message){
        final NormalizedMessage nm = exchange.getMessage(message);
        final List<Property> properties = new ArrayList<Property>();
        final Set<?> set = nm.getPropertyNames();
        final Iterator<?> it = set.iterator();
        while(it.hasNext()){
            final String pName = (String) it.next();
            final Property prop = new Property();
            prop.setName(pName);
            if(nm.getProperty(pName) != null){
                prop.setValue(nm.getProperty(pName).toString());
            } else {
                prop.setValue("null");
            }
            properties.add(prop);
        }
        return properties;
    }
    
    public List<Property> getFaultNMProperties(final MessageExchange exchange){
        final NormalizedMessage nm = exchange.getFault();
        final List<Property> properties = new ArrayList<Property>();
        final Set<?> set = nm.getPropertyNames();
        final Iterator<?> it = set.iterator();
        while(it.hasNext()){
            final String pName = (String) it.next();
            final Property prop = new Property();
            prop.setName(pName);
            if(nm.getProperty(pName) != null){
                prop.setValue(nm.getProperty(pName).toString());
            } else {
                prop.setValue("null");
            }
            properties.add(prop);
        }
        return properties;
    }
    
    public boolean notEmptyProperties(final NormalizedMessage nm){
       return  !nm.getPropertyNames().isEmpty();
    }
    
    public boolean notEmptyProperties(final MessageExchange exchange){
        return !exchange.getPropertyNames().isEmpty();
    }
    
    public boolean notEmptyAttachments(final NormalizedMessage nm){
        return !nm.getAttachmentNames().isEmpty();
    }
    
    public List<String> getNMAttachments(final MessageExchange exchange, final String message){
        final NormalizedMessage nm = exchange.getMessage(message);
        final List<String> attachments = new ArrayList<String>();
        final Set<?> set = nm.getAttachmentNames();
        final Iterator<?> it = set.iterator();
        while(it.hasNext()){
            final String attachmentId = (String) it.next();
            attachments.add(nm.getAttachment(attachmentId).getName());
        }
        return attachments;
    }
    
    public void downloadAttachment(final HttpServletRequest request,final String message, final HttpServletResponse response) throws NonLocalizedError, LocalizedError {
        final String attachmentName = request.getParameter("attachmentName");
        final String exchangeId = request.getParameter("exchangeId");
        final List<MessageExchange> exchanges = this.businessInjector.getExchanges();
        NormalizedMessage nm = null;
        for(final MessageExchange ex : exchanges){
            if(ex.getExchangeId().equals(exchangeId)){
                nm = ex.getMessage(message);
            }
        }
        
        if(nm == null){
            throw new LocalizedError("DOWNLOAD_ATTACHMENT_ERROR"," attachment for message id: "+exchangeId+" not found");
        }
        final Set<?> set = nm.getAttachmentNames();
        final Iterator<?> it = set.iterator();
        while(it.hasNext()){
            final String attachmentId = (String) it.next();
            final DataHandler handler = nm.getAttachment(attachmentId);
            if(handler.getName().equals(attachmentName)){
                response.setContentType(handler.getContentType());
                OutputStream out = null;
                InputStream in = null;
                try {
                    out = response.getOutputStream();
                    in = handler.getInputStream();
                    int readedBytes;
                    in.reset();
                    /*
                     * Write bytes into the the wsdl copy while
                     * readed data is available  (!= -1)
                     */
                    byte[] data = new byte[1024];
                    do{
                    readedBytes = in.read(data);
                        if(readedBytes != -1){
                            out.write(data,0,readedBytes);
                            out.flush();
                        }
                    }while(readedBytes != -1);
                } catch (final IOException e) {
                    throw new NonLocalizedError(DOWNLOAD_ATTACHMENT_ERROR,e.getLocalizedMessage(),e);
                } finally{                   
                    try {
                        out.close();
                        in.close();
                    } catch (final IOException e) {
                        throw new NonLocalizedError(DOWNLOAD_ATTACHMENT_ERROR,e.getLocalizedMessage(),e);
                    }                   
                }                
            }
        }
    }
    
   
    public void  downloadFaultAttachment(final HttpServletRequest request, final HttpServletResponse response) throws NonLocalizedError, LocalizedError {
        final String attachmentName = request.getParameter("attachmentName");
        final String exchangeId = request.getParameter("exchangeId");
        final List<MessageExchange> exchanges = this.businessInjector.getExchanges();
        NormalizedMessage nm = null;
        for(final MessageExchange ex : exchanges){
            if(ex.getExchangeId().equals(exchangeId)){
                nm = ex.getFault();
            }
        }
        
        if(nm == null){
            throw new LocalizedError("DOWNLOAD_ATTACHMENT_ERROR"," attachment for message id: "+exchangeId+" not found");
        }
        final Set<?> set = nm.getAttachmentNames();
        final Iterator<?> it = set.iterator();
        while(it.hasNext()){
            final String attachmentId = (String) it.next();
            final DataHandler handler = nm.getAttachment(attachmentId);
            if(handler.getName().equals(attachmentName)){
                response.setContentType(handler.getContentType());
                OutputStream out = null;
                InputStream in = null;
                try {
                    out = response.getOutputStream();
                    in = handler.getInputStream();
                    int readedBytes;
                    in.reset();
                    /*
                     * Write bytes into the the wsdl copy while
                     * readed data is available  (!= -1)
                     */
                    byte[] data = new byte[1024];
                    do{
                    readedBytes = in.read(data);
                        if(readedBytes != -1){
                            out.write(data,0,readedBytes);
                            out.flush();
                        }
                    }while(readedBytes != -1);
                } catch (final IOException e) {
                    throw new NonLocalizedError(DOWNLOAD_ATTACHMENT_ERROR,e.getLocalizedMessage(),e);
                } finally{                   
                    try {
                        out.close();
                        in.close();
                    } catch (final IOException e) {
                        throw new NonLocalizedError(DOWNLOAD_ATTACHMENT_ERROR,e.getLocalizedMessage(),e);
                    }                   
                }                
            }
        }
    }
    
    public List<String> getFaultNMAttachments(final MessageExchange exchange){
        final NormalizedMessage nm = exchange.getFault();
        final List<String> attachments = new ArrayList<String>();
        final Set<?> set = nm.getAttachmentNames();
        final Iterator<?> it = set.iterator();
        while(it.hasNext()){
            final String attachmentId = (String) it.next();
            attachments.add(nm.getAttachment(attachmentId).getName());
        }
        return attachments;
    }
    
    public String getFreeMemory(){
        return ManagementFactory.getMemoryMXBean().getHeapMemoryUsage().toString();
    }
    
    public boolean checkManually(){
        return this.manually;
    }

    /**
     * @return the rmiServer
     */
    public String getRmiServerName() {
        return this.rmiServer.getHost();
    }
    
    
}
