/**
 * 
 */
package org.ow2.petals.tools.webconsole.uibeans;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.error.NonLocalizedError;
import org.ow2.petals.tools.webconsole.business.ServerFeatureBBean;
import org.ow2.petals.tools.webconsole.util.PetalsConsoleException;
import org.ow2.petals.tools.webconsole.xmlbeans.Server;
import static org.ow2.petals.tools.webconsole.uibeans.ManagementUIBean.PETALS_CONSOLE_ERROR;
/**
 * @author aruffie
 *
 */
public class ServerFeatureUIBean {
    
    private boolean writeServer;
    private String host = "localhost";
    private String port = "7700";
    private String login = "petals";
    private String password = "petals";
    private ServerFeatureBBean businessBean = null;
    private String modifiedHost = "localhost";
    
    public ServerFeatureUIBean() throws NonLocalizedError{
        try {
            businessBean = new ServerFeatureBBean();
        } catch (final PetalsConsoleException e) {
            throw new NonLocalizedError(PETALS_CONSOLE_ERROR,e.getLocalizedMessage(),e);
        }
    }
    
    public boolean isWriteServer() {
        return writeServer;
    }
    
    public void load(final HttpServletRequest request){
        final String selectedHost = request.getParameter("selectedHost");
        for(final Server server : businessBean.getConfiguredServers()){
            if(server.getHost().equals(selectedHost)){
                this.host = server.getHost();
                this.port = server.getJmxPort();
                this.login = server.getJmxLogin();
                this.password = server.getJmxPassword();
                this.modifiedHost = server.getHost();
            }
        }
    }

    public void setWriteServer(boolean writeServer) {
        this.writeServer = writeServer;
    }

    public void saveServer(final String host,final String port,
                           final String login,final String password) throws NonLocalizedError {
        try {
            businessBean.addServer(host,port,login,password);
        } catch (final PetalsConsoleException e) {
            throw new NonLocalizedError(PETALS_CONSOLE_ERROR,e.getLocalizedMessage(),e);
        }
    }
    
    public String performServerForm(final HttpServletRequest request) throws NonLocalizedError{
     
        final String actionType = request.getParameter("actionType");
        
        if(actionType.equals(ServerFeatureBBean.ACTION_TYPE_MODIFY)){
            try {
                businessBean.modifyServer(host,port,login,password,modifiedHost);
            } catch (final PetalsConsoleException e) {
                throw new NonLocalizedError(PETALS_CONSOLE_ERROR,e.getLocalizedMessage(),e);
            }
        }else if(actionType.equals(ServerFeatureBBean.ACTION_TYPE_ADD)){
            try {
                businessBean.addServer(host,port,login,password);
            } catch (final PetalsConsoleException e) {
                throw new NonLocalizedError(PETALS_CONSOLE_ERROR,e.getLocalizedMessage(),e);
            }
        }
        final ManagementUIBean managementUIBean = (ManagementUIBean) request.getSession().getAttribute("managementUIBean");
        if(managementUIBean != null){
            managementUIBean.setHost(host);
            managementUIBean.setPort(port);
            managementUIBean.setLogin(login);
            managementUIBean.setPassword(password);
            managementUIBean.updateDomain();
        }
        return "success";
    }

    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public String getPort() {
        return port;
    }

    public void setPort(String port) {
        this.port = port;
    }

    public String getLogin() {
        return login;
    }

    public void setLogin(String login) {
        this.login = login;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }   
    
    public List<Server> getConfiguredServers(){
        return businessBean.getConfiguredServers();
    }
    
    public void deleteServer(final HttpServletRequest request) throws NonLocalizedError{
        final String host = request.getParameter("host");
        final String port = request.getParameter("port");
        final String login = request.getParameter("login");
        final String password = request.getParameter("password");
        try {
            businessBean.deleteServer(host,port,login,password);
        } catch (final PetalsConsoleException e) {
            throw new NonLocalizedError(PETALS_CONSOLE_ERROR,e.getLocalizedMessage(),e);
        }
    }
}
