/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.util;

import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlFactory;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription.WSDLVersionConstants;
import org.ow2.petals.tools.webconsole.services.PetalsServiceException;
import org.ow2.petals.tools.webconsole.services.management.ManagementService;
import org.ow2.petals.tools.webconsole.services.management.ManagementServiceFactory;
import org.w3c.dom.Document;

/**
 * @author aruffie
 *
 */
public class GeneralHelper {

    public static final String ENDPOINTNAMEPROVIDER = "ENDPOINTNAMEPROVIDER";
    public static final String INTERFACENAMEPROVIDER = "INTERFACENAMEPROVIDER";
    public static final String INTERFACENAMENSURLPROVIDER = "INTERFACENAMENSURLPROVIDER";
    public static final String ADDRESS = "ADDRESS";
    public static final String WSDLVERSION = "WSDLVERSION";
    public static final String NAMESPACEPROVIDER = "NAMESPACEPROVIDER";
    public static final String REPOSITORYPROPERTIE = "managementconsole.repository.dir";
    
    public static String repoPath = null;
    /**
     * help to write a xml document
     * into the specified file
     * @param doc
     * @param file
     */
    public static void writeXmlFile(Document doc, File file) {
        try {
            // Prepare the DOM document for writing
            Source source = new DOMSource(doc);
    
            // Prepare the output file
            Result result = new StreamResult(file);
    
            // Write the DOM document to the file
            Transformer xformer = TransformerFactory.newInstance().newTransformer();
            xformer.transform(source, result);
        } catch (TransformerConfigurationException e) {
        } catch (TransformerException e) {
        }
    }
    
    /**
     * help to recover the ip address
     * by providing a hostname
     * @param addr
     * @return
     * @throws UnknownHostException
     */
    public static String findHostAddr(String host) throws UnknownHostException{
        String[] subString = host.split("/");

        InetAddress inetAddr = null;
        String addrport = subString[2];
        Pattern pattern = Pattern.compile(":");
        Matcher matcher = pattern.matcher(addrport);
        if(matcher.find()){
            String hostname = (String) addrport.subSequence(0, addrport.indexOf(":"));
            pattern = Pattern.compile("([0-9]|[0-9]|[0-9]).([0-9]|[0-9]|[0-9]).([0-9]|[0-9]|[0-9]).([0-9]|[0-9]|[0-9])");
            matcher = pattern.matcher(hostname);
            if(!matcher.find()){
                inetAddr = InetAddress.getByName(hostname);
            }
        }
        if(inetAddr != null){
            return inetAddr.getHostAddress();
        } else {
            return null;
        }
    }
    
    /**
     * try to replace possible "localhost" in the URL by the IP address *
     * @return
     * @throws IOException
     */
    public static String replaceLocalHostByIP() throws IOException{
        InetAddress in = InetAddress.getLocalHost();
        String iPaddress = "";
        InetAddress[] all = InetAddress.getAllByName(in.getHostName());
        for (int i = 0; (i < all.length) && (StringHelper.isNullOrEmpty(iPaddress)); i++) {
            InetAddress ia = all[i];
            if (ia.isReachable(5000)) {
                iPaddress = ia.getHostAddress();
            }
        }
        return iPaddress;
    }
    
    public static String replaceHostByIP(final String host) throws UnknownHostException{
        return InetAddress.getByName(host).getHostAddress();
    }
    
    /**
     * Return a map properties for the specified
     * endpointName/serviceName and founded in
     * the specified description
     * Key Map:
     * ENDPOINTNAMEPROVIDER
     * INTERFACENAMEPROVIDER
     * INTERFACENAMENSURLPROVIDER
     * ADDRESS
     * WSDLVERSION
     * NAMESPACEPROVIDER
     * @param description
     * @param endpointName
     * @param serviceName
     * @return
     */
    public static Map<String,String> readDescription(Description description,
                                final String endpointName,
                                final String serviceName) {
        
        Map<String, String> map = new HashMap<String, String>();
        /*
         * Recover all services
         */
        List<Service> servicesList = description.getServices();
        Iterator<Service> serviceIt = servicesList.iterator();
        map.put(NAMESPACEPROVIDER,description.getTargetNamespace());
        while (serviceIt.hasNext()) {
            Service s = serviceIt.next();

            List<Endpoint> endpointsList = s.getEndpoints();
            Iterator<Endpoint> endpointIt = endpointsList.iterator();
            
            while (endpointIt.hasNext()) {
                Endpoint e = endpointIt.next();
                if ((s.getQName().toString().equals(serviceName))
                        && (e.getName().toString().equals(endpointName))) {
                    map.put(ENDPOINTNAMEPROVIDER,endpointName);
                    map.put(INTERFACENAMEPROVIDER,e.getBinding().getInterface()
                            .getQName().getLocalPart().toString());
                    map.put(INTERFACENAMENSURLPROVIDER,e.getBinding().getInterface()
                            .getQName().getNamespaceURI());
                    map.put(ADDRESS,e.getAddress());
                    if (description.getVersion().equals(WSDLVersionConstants.WSDL11)) {
                        map.put(WSDLVERSION,"1.1");
                    } else {
                        if (description.getVersion().equals(WSDLVersionConstants.WSDL20)) {
                            map.put(WSDLVERSION,"2.0");
                        }
                    }

                }
            }

        }
        return map;
    }
    
    /**
     * Function allow to get Description from a wsdl file
     * @param wsdlFile
     * @return
     * @throws WSDLException
     * @throws URISyntaxException
     * @throws IOException 
     */
    public static Description readWSDL(File wsdlFile) throws WSDLException, URISyntaxException, IOException {
        final WSDL4ComplexWsdlReader wsdl4ComplexWsdlReader = WSDL4ComplexWsdlFactory.newInstance().newWSDLReader();
        return wsdl4ComplexWsdlReader.read(wsdlFile.toURI().toURL());
    }
    
    /**
     * Function allow to get Description from a wsdl document
     * @param wsdlFile
     * @return
     * @throws WSDLException
     * @throws URISyntaxException
     * @throws MalformedURLException
     */
    public static Description readWSDL(Document wsdlDocument) throws WSDLException, URISyntaxException, MalformedURLException {
        final WSDL4ComplexWsdlReader wsdl4ComplexWsdlReader = WSDL4ComplexWsdlFactory.newInstance().newWSDLReader();
        return wsdl4ComplexWsdlReader.read(wsdlDocument);
    }
    
    public static String getRepositoryPath() throws IOException{
        final Properties props = new Properties();
        props.load(GeneralHelper.class.getClassLoader().getResourceAsStream("general.properties"));
        return props.getProperty(REPOSITORYPROPERTIE);
    }
    
    /**
     * Return the repository reference file
     * by providing a servletcontext. If
     * the repository isn't exist, a mkdir
     * @param req
     * @return
     * @throws IOException
     * @throws URISyntaxException
     */
    public static File getRepositoryFile(final HttpServletRequest req) throws IOException, URISyntaxException{
        String reqPath = req.getContextPath();
        reqPath = reqPath.substring(reqPath.lastIndexOf("/") + 1);
        final String realContextPath = req.getSession().getServletContext()
        .getRealPath(req.getContextPath());
        final String rootPath = realContextPath.substring(0, realContextPath
        .lastIndexOf(reqPath));
        final String repoPath = rootPath+"/"+getRepositoryPath();
        final File repository = new File(repoPath);
        if(!repository.exists()){
            repository.mkdir();
        }
        GeneralHelper.repoPath = repository.getPath();
        return repository;
    }
    
    /**
     * Return the URL of the repository
     * @param req
     * @return
     * @throws IOException
     * @throws URISyntaxException
     */
    public static URL getRepositoryURL(final HttpServletRequest req) throws IOException, URISyntaxException{
        return getRepositoryFile(req).toURI().toURL();
    }
    
  
    /**
     * Return the wsdl document for 
     * one endpoint/service.
     * @param hostname
     * @param jmxJNDIPort
     * @param jmxLogin
     * @param jmxPassword
     * @param serviceName
     * @param endpointName
     * @return
     * @throws PetalsServiceException 
     */
    public Document getDocWSDLForEndpoint(final String hostname, final Integer jmxJNDIPort,
                                     final String jmxLogin, final String jmxPassword,
                                     final String serviceName,final String endpointName) throws PetalsServiceException{
        ManagementService service = (ManagementService) ManagementServiceFactory.getInstance().getService(hostname, jmxJNDIPort, jmxLogin, jmxPassword);
        return service.getDocument(serviceName, endpointName);
    }
    
    /**
     * Return the wsdl document for
     * one endpoint/service wellformed
     * into a string.
     * @param hostname
     * @param jmxJNDIPort
     * @param jmxLogin
     * @param jmxPassword
     * @param serviceName
     * @param endpointName
     * @return
     * @throws PetalsServiceException 
     */
    public String getStrWSDLForEndpoint(final String hostname, final Integer jmxJNDIPort,
                                     final String jmxLogin, final String jmxPassword,
                                     final String serviceName,final String endpointName) throws PetalsServiceException{
        
        final Document doc = getDocWSDLForEndpoint(hostname, jmxJNDIPort, jmxLogin, jmxPassword, serviceName, endpointName);
        if(doc != null){
            return StringHelper.HTMLEntityEncode(StringHelper.prettyPrint(getDocWSDLForEndpoint(hostname, jmxJNDIPort, jmxLogin, jmxPassword, serviceName, endpointName)));
        } else {
            return "";
        }
    }
    
    /**
     * Return the URI to be used by PEtALS servers to access downloadable
     * artefacts. With http downloads, As the WebConsole can run on multi-IP
     * server, we have to find the correct IP. <br>
     * <li>First, look at the configuration file,</li> <li>then the IP set by
     * web users in their HttpRequest,</li> <li>otherwise, if the HttpRequest IP
     * is localhost, use the JDK InetAddress.getLocalHost().getHostAddress()
     * method</li> <br>
     * For ftp downloads, we retrieve the ftp address from the configuration
     * file
     * 
     * @param httpServletRequest
     * @return
     * @throws URISyntaxException
     */
    public static URI getURI(HttpServletRequest httpServletRequest) throws URISyntaxException {
        // check from httprequest
        URI uri = null;
        if(uri == null){
            uri = getURIFromHttpRequest(httpServletRequest);
        }
        if(uri == null){
            uri = getURIFromInetAddress(httpServletRequest);
        }
        if(uri == null){
            uri = new URI(httpServletRequest.getRequestURI());
        }
        return uri;
    }
    
    /**
     * Retrieve URI from http request : http uri. If host is localhost, return null.
     * httpRequest parameters are removed from the URL
     * @param httpServletRequest
     * @return
     * @throws URISyntaxException
     */
    public static URI getURIFromHttpRequest(HttpServletRequest httpServletRequest) throws URISyntaxException {
        URI uri = new URI(httpServletRequest.getRequestURL().toString());
        if("localhost".equalsIgnoreCase(uri.getHost())){
            uri = null;
        }else{
            uri = new URI(uri.getScheme(),uri.getUserInfo(),uri.getHost(),uri.getPort(),uri.getPath(),null,null);
        }
        return uri;
    }
    
    /**
     * Replace the Host in the httpRequest URL by the IP retreived from JDK InetAddress API 
     * httpRequest parameters are removed from the URL
     * @return
     * @throws URISyntaxException
     */
    public static URI getURIFromInetAddress(HttpServletRequest httpServletRequest) throws URISyntaxException {
        URI uri =null;
        try {
            String ip = InetAddress.getLocalHost().getHostAddress();
            uri = new URI(httpServletRequest.getRequestURL().toString());
            //replace uri host with ip
            uri = new URI(uri.getScheme(),uri.getUserInfo(),ip,uri.getPort(),uri.getPath(),null,null);
        } catch (UnknownHostException e) {
            throw new URISyntaxException("InetAddress.getLocalhost()","URI can not be construct from machine IP information");
        }
        return uri;
    }
    
    /**
     * Allow to delete a uploaded artifact if
     * it's store into the repository
     * @param HttpServletRequest request
     * @param String stringURL
     * @throws IOException
     * @throws URISyntaxException
     */
    public static void deleteUploadedArtifact(final HttpServletRequest request, final String stringURL) throws IOException, URISyntaxException{
        final String[] splittedPart = stringURL.split("/");
        if(splittedPart.length != 0){
            final String fileName = splittedPart[splittedPart.length-1];
            final File repo = GeneralHelper.getRepositoryFile(request);
            final File artifact = fileFounder(fileName, repo);
            if(artifact != null && artifact.exists()){
                artifact.delete();
            }
        }
    }
    
    /**
     * Allow to search a file into a directory
     * @param String fileName
     * @param File directory
     * @return File
     */
    public static File fileFounder(final String fileName, final File directory){
        for(final File f : directory.listFiles()){
            //If the sub file is a directory search into it
            if(f.isDirectory()){
                final File file = fileFounder(fileName, f);
                if(file != null){
                    return file;
                }
            } else {
                if(f.getName().equals(fileName)){
                    return f;
                }
            }
        }
        return null;
    }
    
    /**
     * Allow to delete a file hierarchy
     * @param File tree
     */
    public static void deleteFileTree(final File tree){
        for(final File subFile : tree.listFiles()){
            if(subFile.isDirectory()){
                deleteFileTree(subFile);
            } else {
                subFile.delete();
            }
        }
        tree.delete();
    }
    
    /**
     * Allow to delete a file hierarchy
     * @param File tree
     */
    public static void deleteDirectoryContent(final File tree){
        for(final File subFile : tree.listFiles()){
            if(subFile.isDirectory()){
                deleteFileTree(subFile);
            } else {
                subFile.delete();
            }
        }
    }
}
