/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webconsole.util;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.objectweb.petals.tools.rmi.common.util.InputStreamForker;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

/**
 * @author aruffie
 *
 */
public class StringHelper {

    /**
     * Return true if the String is null or its size is 0.
     * 
     * @param s
     * @return
     */
    public static boolean isNullOrEmpty(final String s) {
        return (s == null) || s.trim().equals("");
    }
    
    /**
     * Return true if the String is not null and its size is > 0.
     * 
     * @param s
     * @return
     */
    public static boolean isNotNullAndNotEmpty(final String s) {
        return !StringHelper.isNullOrEmpty(s);
    }
    
    public static String createString(final Source s) throws TransformerFactoryConfigurationError, IOException, TransformerException{
        String result = null;
        if(s instanceof StreamSource && ((StreamSource)s).getInputStream().markSupported()){
            StreamSource ss = (StreamSource)s;
            ss.getInputStream().reset();
            byte[] buf = new byte[ss.getInputStream().available()];
            BufferedInputStream bis = new BufferedInputStream(ss.getInputStream());
            bis.read(buf);
            result = new String(buf);
            ss.getInputStream().reset();
            bis.close();
        } else {
            StringWriter buffer = new StringWriter();
            Result sresult = new StreamResult(buffer);
            Transformer transform = TransformerFactory.newInstance().newTransformer();
            transform.transform(s, sresult);
            result = buffer.toString();
        }
        return result;
    }
    
    public static String prettyPrintWithSource(final Source s) throws TransformerFactoryConfigurationError, IOException, TransformerException{
        return prettyPrint(createString(s));
    }
    
    /**
     * Allow to fork into two inputstream a Source
     * @param Source s
     * @return InputStreamForker
     * @throws TransformerFactoryConfigurationError
     * @throws IOException
     * @throws TransformerException
     */
    public static InputStreamForker fork(final Source s) throws TransformerFactoryConfigurationError, IOException, TransformerException{
        final String msg = prettyPrintWithSource(s);
        byte[] msgByte = msg.getBytes("UTF-8");
        ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
        final InputStreamForker forker = new InputStreamForker(in);
        return forker;
    }
    
    public static Source createSource(final String msg) throws UnsupportedEncodingException{
        StreamSource source = new StreamSource();
        byte[] msgByte = msg.getBytes("UTF-8");
        ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
        source.setInputStream(in);
        return source;
    }
    
    public static String parseToString(final Node node) throws TransformerFactoryConfigurationError, TransformerException{
        String result = null;
        if(node != null){
            node.normalize();
            Transformer transformer = TransformerFactory.newInstance().newTransformer();
            StringWriter stringWriter = new StringWriter();
            transformer.transform(new DOMSource(node), new StreamResult(stringWriter));
            StringBuffer buffer = stringWriter.getBuffer();
            result = buffer.toString();
        }
        return result;
    }

    /**
     * parse the xml String and return it pretty-printed (with correct
     * indentations, etc..) If an error occurs during the process, the returned
     * String will be the original XML string
     * 
     * @param xmlResponse
     *            the xml String to pretty print
     * @return pretty printed string if no error occurs. If an error occurs,
     *         return the original string
     */
    public static String prettyPrint(String xmlResponse) {
        String result = xmlResponse;
        try {
            /** parse xml string as a dom tree * */
            final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.newDocumentBuilder();
            DocumentBuilder parser;
            parser = factory.newDocumentBuilder();
            final InputSource is = new InputSource(new StringReader(xmlResponse));
            final Document document = parser.parse(is);

            result = StringHelper.prettyPrint(document);

        } catch (final Exception e) {
            // if an error occurs, the result will be the original string
        }
        return result;
    }
    
    /**
     * parse the xml String and return it pretty-printed (with correct
     * indentations, etc..) If an error occurs during the process, the returned
     * String will be the original XML string
     * 
     * @param xmlResponse
     *            the xml String to pretty print
     * @returnpretty printed string if no error occurs. If an error occurs,
     *               return the original string
     */
    public static String prettyPrint(final Document xmlDocument) {
        String result = "";
        try {
            ByteArrayOutputStream outStream = new ByteArrayOutputStream();
            XMLPrettyPrinter.prettify(xmlDocument, outStream);
            result = outStream.toString("UTF-8");
        } catch (final Exception e) {
            // if an error occurs, the result will be the original string
        }
        return result;
    }

    /**
     * Encode a String in HTML format
     * 
     * @param s
     *            the String to encode
     * @return a html encoded string
     */
    public static String HTMLEntityEncode(String response) {
        final StringBuffer buf = new StringBuffer();
        for (int i = 0; i < response.length(); i++) {
            final char c = response.charAt(i);
            if (c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z' || c >= '0' && c <= '9') {
                buf.append(c);
            } else if ((c == '\r') && (response.charAt(i + 1) == '\n')) {
                buf.append("<br />");
            } else if (c == ' ') {
                buf.append("&nbsp;");
            } else {
                buf.append("&#" + (int) c + ";");
            }
        }
        return buf.toString();
    }
}
